chrome.runtime.onMessage.addListener(function (request, sender, sendResponse) {
    let data = request.data || request.data === 0 ? request.data : {};
    let categories = request.categories ? request.categories : {};
    let response = {asyncResponse: false, sender: sender, request: request};

    //Actions
    if (detectReceiver(sender) === 'extension') {
        switch (request.action) {
            case 'delete_cookie':
                new Promise(async function (resolve, reject) {
                    let tab = await getCurrentTab();
                    chrome.cookies.remove({"url": tab.url, "name": "gdpr-accepted"}, function(deleted_cookie) {
                        send.browser({action: 'reload_page'});
                    });
                }).then(function (result) {
                    send.extension({action: request.action, data: result});
                });
                break;
            case 'set_auto_accept_when_all_known':
                chrome.storage.sync.set({autoAcceptWhenKnown: data}, function () {});
                send.browser({action: 'add_and_load_accepted_gdpr_options', data: {autoAcceptWhenKnown: data}});
                break;
            case 'set_auto_accept_all':
                chrome.storage.sync.set({autoAccept: data}, function () {});
                send.browser({action: 'add_and_load_accepted_gdpr_options', data: {autoAcceptWhenKnown: data}});
                break;
            case 'debug_clear_storage':
                clearStorage();
                break;
            case 'get_tab_gdpr_options':
                new Promise(async function (resolve, reject) {
                    resolve({
                        tab: await getCurrentTab(),
                        gdpr: await getOptionCollection(),
                        categories: await getCategoryCollection(),
                        autoAcceptWhenKnown: await getAutoAcceptWhenKnown(),
                        autoAccept: await getAutoAccept(),
                        language: navigator.language
                    });
                }).then(function (result) {
                    send.extension({action: request.action, data: result});
                });
                response.asyncResponse = true;
                break;
            case 'update':
                updateCategories(categories);
                update(data);
                response.asyncResponse = true;
                let items = {};

                for(let keyName in data) {
                    let item = data[keyName];
                    items[keyName] = {
                        enabled: item.enabled,
                        required: item.required
                    }
                }
                send.browser({action: 'add_and_load_accepted_gdpr_options', data: {items: items}});
                break;
        }
    } else if (detectReceiver(sender) === 'browser') {
        switch (request.action) {
            case 'add_and_load_accepted_gdpr_options':
                new Promise(async function (resolve, reject) {
                    let result = await getOptionCollection();

                    let response = {};
                    for (let keyName in result) {
                        if (result.hasOwnProperty(keyName)) {
                            let host = sender.url.replace('http://', '').replace('https://', '');
                            if ((host.length - 1) === host.lastIndexOf('/')) {
                                host = host.substr(0, host.length - 1);
                            }
                            if (matchDomain(host, result[keyName].domain) && result[keyName].domain[host].enabled) {
                                response[keyName] = {
                                    enabled: result[keyName].domain[host].enabled,
                                    required: result[keyName].domain[host].required
                                }
                            }
                        }
                    }

                    resolve({
                        autoAcceptWhenKnown: await getAutoAcceptWhenKnown(),
                        autoAccept: await getAutoAccept(),
                        items: response
                    });
                }).then(function (result) {
                    send.browser({action: 'add_and_load_accepted_gdpr_options', data: result});
                });


                response.asyncResponse = true;
                updateCategories(categories);
                insert(data);
                break;
            case 'update':
                send.browser({action: 'update', data: request});
                updateCategories(categories);
                update(data);
                response.asyncResponse = true;
                break;
        }
    }

    sendResponse(response);
});

function DEBUG(data, src_action, trace) {
    console.log(data);
    console.trace();
    //send.browser({action: 'debug', data:data, src_action:src_action, trace:trace});
}

let send = {
    browser: async function (data) {
        let tab = await getCurrentTab();
        try {
            chrome.tabs.sendMessage(tab.id, data);
        } catch (e) {
            DEBUG(e);
        }
    },
    extension: function (data, callback) {
        callback = callback ? callback : function () {
        };
        chrome.runtime.sendMessage(data, callback);
    }
};

function detectReceiver(sender) {
    if (sender.tab) {
        return 'browser';
    } else {
        return 'extension';
    }
}

function getCurrentBrowserTab(callback) {
    let params = {
        active:true,
        currentWindow: true
    };
    chrome.tabs.query(params, callback)
}

function matchDomain(tabDomain, domainCollection) {
    for (let domainName in domainCollection) {
        if (domainCollection.hasOwnProperty(domainName) && tabDomain.match(domainName)) {
            return true;
        }
    }

    return false;
}

function updateCategories(list) {
    chrome.storage.sync.get(['gdprCategoryList'], function (result) {
        result = result.gdprCategoryList;
        if (result === undefined) {
            result = {};
        }

        for (let keyName in list) {
            result[keyName] = list[keyName];
        }

        try {
            chrome.storage.sync.set({gdprCategoryList: result}, function (value) {
                DEBUG(value);
            });
        } catch (e) {
            DEBUG(e);
        }
    });
}

function insert(list) {
    update(list, true)
}

function update(list, onlyInsert) {
    chrome.storage.sync.get(['gdprOptionList'], function (result) {
        result = result.gdprOptionList;
        if (result === undefined) {
            result = {};
        }

        let sendDomain = null;
        //Save new Keys in extension
        for (let keyName in list) {
            if (list.hasOwnProperty(keyName)) {
                let keyData = list[keyName];
                sendDomain = keyData.domain;
                if (result.hasOwnProperty(keyName)) {
                    if ((result[keyName].domain[sendDomain] === undefined && onlyInsert) || !onlyInsert) {
                        result[keyName].domain[sendDomain] = {
                            enabled: keyData.enabled,
                            required: keyData.required
                        }
                    }
                } else {
                    let domainSettings = {};
                    domainSettings[sendDomain] = {
                        enabled: keyData.enabled,
                        required: keyData.required
                    };
                    result[keyName] = {
                        label: keyData.label,
                        category: keyData.category,
                        domain: domainSettings
                    }
                }
            }
        }

        //delete Old keys in extension
        for (let keyName in result) {
            if (list.hasOwnProperty(keyName)) {
                for (let domainName in result[keyName].domain) {
                    if (result[keyName].domain.hasOwnProperty(domainName)) {
                        if (domainName !== sendDomain && list.hasOwnProperty(keyName)) {
                            delete result[keyName].domain[domainName];
                            break;
                        }
                    }
                }
            }
        }

        try {
            chrome.storage.sync.set({gdprOptionList: result}, function (value) {
                DEBUG(value);
            });
        } catch (e) {
            DEBUG(e);
        }
    });
}

function clearStorage() {
    console.log("Clear Storage");
    chrome.storage.sync.set({gdprOptionList: {}}, function (value) {
    });
    chrome.storage.sync.set({gdprCategoryList: {}}, function (value) {
    });
}