let container = {};
function main() {
    send('get_tab_gdpr_options', build);
}

main();


function build(data) {
    container = data;
    renderContext();
    renderCurrentPage();
    renderSettings();
    addListener();
}

function renderContext() {
    document.getElementById('extension-context').innerHTML = render({}, 'main_view');
    document.getElementById('clear_storage').onclick = function() {
        send('debug_clear_storage', function() {});
    };

    document.getElementById('delete_cookie').onclick = function() {
        send('delete_cookie', function() {});
    };

    document.getElementById('go_to_orcas').onclick = function() {
        chrome.tabs.create({ url: 'https://www.orcas.de' });
    };

    document.getElementById('auto-accept-every-time-all').onclick = function() {
        send('set_auto_accept_all', this.checked ? 1 : 0,  function() {});
    };

    document.getElementById('auto-accept-when-all-selected').onclick = function() {
        send('set_auto_accept_when_all_known', this.checked ? 1 : 0, function() {});
    };
}

function renderSettings() {
    document.getElementById('auto-accept-every-time-all').checked = container.autoAccept;
    document.getElementById('auto-accept-when-all-selected').checked = container.autoAcceptWhenKnown;
}

function renderCurrentPage(data) {
    let content = '';
    let options = container.gdpr;
    let domain = container.tab && container.tab.url ? getDomain(container.tab.url) : '';

    for(let optionName in options) {
        if(options.hasOwnProperty(optionName)) {
            let currentPageData = {
                category: getCategory(options[optionName].category),
                domain_name: domain.split('.').join('-'),
                domain:domain,
                name: optionName,
                label: options[optionName].label,
                enabled: options[optionName].domain[domain] && options[optionName].domain[domain].enabled ? 'checked' : '',
                disabled: options[optionName].domain[domain] && options[optionName].domain[domain].required ? 'disabled' : '',
                readonly: options[optionName].domain[domain] && options[optionName].domain[domain].required ? 'readonly' : 'readable',
                classes: options[optionName].domain[domain] ? '' : 'hide-item'
            };

            content += render(currentPageData, 'gdpr_options');
        }
    }

    document.getElementById('gdpr-option-list').innerHTML = content;
}

function getCategory(slug) {
    for(let cat in container.categories) {
        if(container.categories[cat].category === slug) {
            return container.categories[cat].label;
        }
    }

    return 'Unknown';
}

function addListener() {
    let collection = document.getElementsByClassName('readable');
    for(let ii = 0; ii < collection.length; ii++) {
        collection[ii].onclick = function() {
            let data = {};
            let id = this.getAttribute('data-name');
            let itemDomain = this.getAttribute('data-domain');
            data[id] = {
                domain: itemDomain,
                enabled: this.checked,
                required: false
            };

            send('update', data, function(response) {});

            if(itemDomain) {
                itemDomain = itemDomain.split('.').join('-');
                let dom = document.getElementById(itemDomain + '-' + id);
                if(dom.getAttribute('data-domain') === itemDomain) {
                    dom.checked = this.checked;
                }
            }
        }
    }
}